<?php

namespace Drupal\translators\Services;

use Drupal\Component\Utility\Html;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Class TranslatorSkills.
 *
 * Translator skills helper class.
 *
 * @package Drupal\translators\Services
 */
class TranslatorSkills {
  use StringTranslationTrait;
  /**
   * The content translation manager.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;
  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;
  /**
   * Language skills field name.
   *
   * @var string
   */
  protected $fieldName;
  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;
  /**
   * Current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request|null
   */
  protected $request;
  /**
   * Messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;
  /**
   * Entity field.
   *
   * @var \Drupal\Core\Field\FieldItemList|null
   */
  protected $entityField;

  /**
   * Constructs a TranslatorSkills service.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   Current user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager.
   * @param \Symfony\Component\HttpFoundation\RequestStack $stack
   *   Request stack.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Messenger.
   */
  public function __construct(
    AccountInterface $current_user,
    ConfigFactoryInterface $config_factory,
    EntityTypeManagerInterface $entity_type_manager,
    RequestStack $stack,
    MessengerInterface $messenger
  ) {
    $this->currentUser       = $current_user;
    $this->configFactory     = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->request           = $stack->getCurrentRequest();
    $this->messenger         = $messenger;
    $this->fieldName         = $this->configFactory
      ->get('translators.settings')
      ->get('translation_skills_field_name');
    if (!$this->fieldName === NULL && !$this->currentUser->id() === NULL) {
      $this->loadTranslationSkills('user', $this->fieldName, $this->currentUser->id());
    }
  }

  /**
   * Get array with translation skills langcodes.
   *
   * @return array
   *   Multi-level array of all the translation skill langcodes.
   */
  public function getTranslationSkills() {
    $this->loadTranslationSkills('user', $this->fieldName, $this->currentUser->id());
    return $this->entityField ? $this->entityField->getValue() : [];
  }

  /**
   * Get source langcodes.
   *
   * @return array
   *   Array of all the source langcodes.
   */
  public function getSourceLangcodes() {
    $this->loadTranslationSkills('user', $this->fieldName, $this->currentUser->id());
    return $this->entityField ? $this->entityField->getValueSourceLangcodes() : [];
  }

  /**
   * Get target langcodes.
   *
   * @return array
   *   Array of all the source langcodes.
   */
  public function getTargetLangcodes() {
    $this->loadTranslationSkills('user', $this->fieldName, $this->currentUser->id());
    return $this->entityField ? $this->entityField->getValueTargetLangcodes() : [];
  }

  /**
   * Get all langcodes.
   *
   * @return array
   *   Array of all the source langcodes.
   */
  public function getAllLangcodes() {
    $this->loadTranslationSkills('user', $this->fieldName, $this->currentUser->id());
    return $this->entityField ? $this->entityField->getValueAllLangcodes() : [];
  }

  /**
   * Check if user has a specified language skill.
   *
   * @param string $langcode
   *   Language ID.
   *
   * @return bool
   *   TRUE - if a specified language skill exists in users skills list,
   *   FALSE otherwise.
   */
  public function hasLangcode($langcode) {
    return in_array($langcode, $this->getAllLangcodes());
  }

  /**
   * Show empty message.
   *
   * @param string|int|null $user_id
   *   User ID.
   */
  public function showMissingTranslationSkillsWarning($user_id = NULL) {
    $enabled = $this->configFactory
      ->get('translators.settings')
      ->get('enable_missing_skills_warning');
    // Prevent showing warning message
    // if the appropriate feature is not enabled.
    if (empty($enabled)) {
      return;
    }
    $user_id  = !$user_id ? $this->currentUser->id() : $user_id;
    $options  = $this->getEditTranslationSkillLinkWithRedirect();
    $edit_url = Url::fromRoute('entity.user.edit_form', ['user' => $user_id], $options);
    $link     = Link::fromTextAndUrl('here', $edit_url)->toString();
    $message  = $this->t("Please register your translation skills @here", ['@here' => $link]);
    // Prevent duplicated warning messages on a page.
    $messages = $this->messenger->messagesByType('warning');
    if (empty($messages) || !in_array($message, $messages)) {
      $this->messenger->addWarning($message);
    }
  }

  /**
   * Build edit link options.
   *
   * @return array
   *   Edit link options array.
   */
  protected function getEditTranslationSkillLinkWithRedirect() {
    $options = [];
    // Add fragment option to get user
    // automatically scrolled down to the needed field.
    $options['fragment'] = 'edit-' . Html::getId($this->fieldName) . '-wrapper';
    // Add destination URL to get user being redirected back
    // to the correct page after saving the user's edit form.
    $options['query'] = [
      'destination' => $this->request->getRequestUri(),
    ];
    return $options;
  }

  /**
   * Load translation skills for current user.
   *
   * @param string $entity_type
   *   Entity type ID.
   * @param string $field_name
   *   Field name ID.
   * @param int|string $id
   *   Entity ID.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function loadTranslationSkills(string $entity_type, string $field_name, $id) {
    $this->entityTypeManager->getStorage($entity_type)->resetCache([$id]);
    $entity = $this->entityTypeManager->getStorage($entity_type)->load($id);
    if (isset($entity->{$field_name})) {
      $this->entityField = $entity->{$field_name};
    }
  }

}
